import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import java.io.*;
import java.util.Vector;
import java.util.StringTokenizer;

import com.jinsight.jetchart.*;

/**
 * This demo displays a series  that  has  its  values updated against time. It is a strip chart 
 * that periodically retrieves values from a data source, in this case a text file, and refreshes
 * data points. The refresh rate  and the maximum number of data points simultaneously displayed
 * can be set in the bottom panel.
 *
 * @author Andre de Lima Soares - alsoares@jinsight.com
 */
public class Main extends JFrame implements Runnable,ActionListener,AdjustmentListener,ItemListener {

    Graph graph;
    GraphSerie gs;
    JButton b1,b2;
    JScrollBar sb1,sb2;
    JLabel lb1,lb2;
    JComboBox series;
    boolean loop;
    long interval=500;
    int maxNrOfLabels=100;

    // This object is used to synchronize access to critical code.
    Object lock=new Object();

    public Main() {

	setTitle("The JetChart Library - A strip chart demo");

        addWindowListener(new WindowAdapter() {
           public void windowClosing(WindowEvent evt) {
              System.exit(0);
           }
        });

	// Creates the top and bottom panels.
	JPanel topPanel=new JPanel(new FlowLayout(FlowLayout.LEFT));
	JPanel bottomPanel=new JPanel(new FlowLayout(FlowLayout.LEFT));

	// Creates components to be displayed in the top and bottom panels.
	sb1=new JScrollBar(JScrollBar.HORIZONTAL,500,1,10,1001);
        sb1.setPreferredSize(new Dimension(80,sb1.getPreferredSize().height));
        sb1.addAdjustmentListener(this);

	sb2=new JScrollBar(JScrollBar.HORIZONTAL,100,1,5,251);
        sb2.setPreferredSize(new Dimension(80,sb1.getPreferredSize().height));
        sb2.addAdjustmentListener(this);

	b1=new JButton("Start");
	b2=new JButton("Stop");
	b2.setEnabled(false);
	b1.addActionListener(this);
	b2.addActionListener(this);

        String[] items={"Line series","Area series","Bar series"};
        series=new JComboBox(items);
	series.addItemListener(this);

	lb1=new JLabel("500");
	lb2=new JLabel("100");

	// Adds the top and bottom Panel.
	topPanel.add(b1);
	topPanel.add(b2);
	topPanel.add(new JLabel("Series type:"));
	topPanel.add(series);

	bottomPanel.add(new JLabel("Speed(10-1000 ms):"));
	bottomPanel.add(sb1);
	bottomPanel.add(lb1);
	bottomPanel.add(new JLabel("Max nr of data points:"));
	bottomPanel.add(sb2);
	bottomPanel.add(lb2);

        Container ct=getContentPane();
	ct.add("North",topPanel);
	ct.add("South",bottomPanel);

	// Creates the chart context and sets its properties.
	graph=new Graph();

        // Disable labels marks
        graph.getGraphSet(0).setLabelsMarksEnabled(false);
              
	// Enables the wall of the primary GraphSet. See JetChart documentation
	// about the GraphSet class for further details.
	Wall wall=graph.getGraphSet(0).getWall();
	wall.setEnabled(true);
	wall.setColor(new Color(204,255,255));

	// Disables automatic scale and sets max and min scale values.
	Scale scale=graph.getGraphSet(0).getScale();
	scale.setAutoScaleEnabled(false);
	scale.setMaxValue(1);
	scale.setIncrement(.2);

	// Enables grid.
	Grid grid=graph.getGraphSet(0).getGrid();
	grid.setEnabled(true);
	grid.setColor(Color.lightGray);
				
	ct.add("Center",graph);

	setSize(550,300);

	setVisible(true);

    }

    public void itemStateChanged(ItemEvent evt) {
	
        // Sets the GraphSerie reference to null to force the thread to recreate the series.
	// If the timer thread is running and updating chart, the thread holds a lock on 
	// the Lock object, preventing series from being removed and chart from being refreshed
	// concurrently.
	synchronized (lock) {
	    graph.removeSerie(gs);
	    gs=null;
	    graph.refresh();
	}
    }

    public void adjustmentValueChanged(AdjustmentEvent evt) {
	if (evt.getSource()==sb1) {

	    // Sets the refresh time
	    interval=sb1.getValue();
	    lb1.setText(Long.toString(interval));
	}
	else {

	    // Sets the maximum number of data points simultaneously displayed.
	    maxNrOfLabels=sb2.getValue();
	    lb2.setText(Integer.toString(maxNrOfLabels));
	}
    }

    public void actionPerformed(ActionEvent evt) {
	
	// Starts thread.
	if (evt.getSource()==b1) {
	    b1.setEnabled(false);
	    b2.setEnabled(true);
	    sb2.setEnabled(false);
	    Thread t=new Thread(this);
	    t.start();
	}
	else {
	    b1.setEnabled(true);
	    b2.setEnabled(false);
	    sb2.setEnabled(true);
	    loop=false;
	}
    }

    public void run() {

	try {

	    // This flag controls loop end.
	    loop=true;

	    // Creates the input stream to fetch values and labels.
	    File f1=new File("values.txt");
	    BufferedReader br1=new BufferedReader(new FileReader(f1));
	    String line1=null;

	    // These Vectors keep the values and labels read from the text file.
	    Vector values=new Vector();
	    Vector labels=new Vector();
            
            // Creates the first  data points to be displayed. We want the
            // chart to display nothing at the first time, so we set the first
            // data points with value 0 and empty label.
            for (int counter=0;counter<maxNrOfLabels;counter++) {
               values.addElement(new Double(0));
               labels.addElement("");
            }

            // Chart must start painting the data point of a series located at 
            // the 'maxNrOfLabels' position.
            // This variable is decreased at the end of the loop below.
            int startPoint=maxNrOfLabels;

	    // Starts the timer.
	    while (loop) {
		
		// Sleeps for 'interval' milisseconds.
		Thread.sleep(interval);

		line1=br1.readLine();
		
		// The following block is synchronized on the Lock object to prevent concurrent
		// chart refresh, since the 'itemStateChanged' handler might be invoked at any
		// time, also causing current series to be removed and consequently leading to a 
		// possible null pointer exception.
		synchronized (lock) {

		    if (line1!=null) {
			
			// If the number of  values read exceeds the maximum number of data
			// points to be displayed, the first value in the 'values' Vector
			// is removed, as well as the first label. 
			if (values.size()>=maxNrOfLabels) {
			    values.removeElementAt(0);
			    labels.removeElementAt(0);
			}
			
			
			// Adds the new value and label. The chart starts moving to the left
			// when the first value and label are removed as described above and
			// the new ones are added.
			values.addElement(Double.valueOf(line1));
			labels.addElement("");
			
			// Creates the arrays to keep the values and labels. The array of 
			// double values is passed to the series, and the array of labels
			// is passed to the Graph object.
			double[] arrayValues=new double[values.size()];
			String[] arrayLabels=new String[labels.size()];
			
			// Starts filling the arrays.
			for (int counter=0;counter<values.size();counter++) {
			    double d=((Double)values.elementAt(counter)).doubleValue();
			    String s=(String)labels.elementAt(counter);
			    arrayValues[counter]=d;
			    arrayLabels[counter]=s;
			}
			
			// Creates series, if it wasn't done yet.
			if (gs==null) {
			    int type=series.getSelectedIndex();
			    if (type==0) {
				gs=new LineSerie(arrayValues,"");
				gs.setMarksEnabled(false);
			    }
			    else if (type==1) {
				gs=new AreaSerie(arrayValues,"");
				((AreaSerie)gs).setAreaLinesEnabled(false);
			    }
			    else if (type==2) {
				gs=new BarSerie(arrayValues,"");
				((BarSerie)gs).setWidth(1);
				((BarSerie)gs).setBorderEnabled(false);
			    }
			    gs.setColor(Color.red);
			    graph.addSerie(gs);
			}
			else 
			    // Sets series values.
			    gs.setValues(arrayValues);
			
			
			gs.setStartPoint(startPoint-1);
			if (startPoint>0)
			    startPoint--;
			// Sets chart labels.
			graph.setLabels(arrayLabels);
			
			// Refreshes chart.
			graph.repaint();
			
		    }
		    // After reading all lines from the text file, ends the loop.
		    else {
			loop=false;
		    }
		    
		}

	    }
	    
	}
	
	catch (Exception e) {
	    e.printStackTrace();
	}
	finally {
	    // At the end of thread execution, resets components to their initial state.
	    resetComponents();
	}
	
    }

    private void resetComponents() {
	b1.setEnabled(true);
	b2.setEnabled(false);
	sb2.setEnabled(true);
    }

    public static void main(String[] args) {
	Main main=new Main();
    }
}
